from appchance.multi_db import db_config


class MultiDatabaseRouter(object):
    """
    Router for multi-database app.
    """

    def db_for_read(self, model, **hints):
        if hasattr(db_config, "db"):
            return db_config.db
        return "default"

    def db_for_write(self, model, **hints):
        if hasattr(db_config, "db"):
            return db_config.db
        return "default"

    def allow_relation(self, obj1, obj2, **hints):
        return True

    def allow_migrate(self, db, app_label, model_name=None, **hints):
        return True



from rest_framework import serializers


class CategorySerializer(serializers.Serializer):
    id = serializers.IntegerField()
    name = serializers.CharField()
    childs = serializers.SerializerMethodField()

    @classmethod
    def get_childs(cls, obj):
        return cls(obj.category_set.all(), many=True).data or None


class CategoryRecursiveSerializer(serializers.Serializer):
    id = serializers.IntegerField()
    name = serializers.CharField()
    parent = serializers.SerializerMethodField()

    @classmethod
    def get_parent(cls, obj):
        if obj.parent:
            return cls(obj.parent).data



"""
Django settings for integration_test project.
Generated by 'django-admin startproject' using Django 1.10.5.
For more information on this file, see
https://docs.djangoproject.com/en/1.10/topics/settings/
For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.10/ref/settings/
"""

import os

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.10/howto/deployment/checklist/

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []

SECRET_KEY = "secret"

# Application definition

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "appchance.sections",
    "sections_app",
]

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
]

ROOT_URLCONF = "config.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]

WSGI_APPLICATION = "integration_test.wsgi.application"

# Database
# https://docs.djangoproject.com/en/1.10/ref/settings/#databases

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": os.path.join(BASE_DIR, "db.sqlite3"),
    }
}

# Internationalization
# https://docs.djangoproject.com/en/1.10/topics/i18n/

LANGUAGE_CODE = "en-us"

TIME_ZONE = "UTC"

USE_I18N = True

USE_L10N = True

USE_TZ = True

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.10/howto/static-files/

STATIC_URL = "/static/"

# django-rest-framework
# https://www.django-rest-framework.org/api-guide/settings/
# -------------------------------------------------------------------------------
REST_FRAMEWORK = {
    "DEFAULT_AUTHENTICATION_CLASSES": (
        "rest_framework.authentication.SessionAuthentication",
        "rest_framework.authentication.TokenAuthentication",
    ),
    "DEFAULT_PERMISSION_CLASSES": ("rest_framework.permissions.IsAuthenticated",),
}


PYCHANCE_SECTIONS_CONTENT_MODEL = "sections_app.CustomContent"



import json
import requests
from urllib3.util import Retry
from requests.adapters import HTTPAdapter
from requests.exceptions import ConnectionError


class HttpClient:
    """API client which expects JSON response."""

    def __init__(
        self, retries=5, backoff_factor=0.1, retry_on_codes=(500, 502, 503, 504)
    ):
        retry = Retry(
            total=retries,
            read=retries,
            connect=retries,
            backoff_factor=backoff_factor,
            status_forcelist=retry_on_codes,
        )
        adapter = HTTPAdapter(max_retries=retry)
        self.session = requests.Session()
        self.session.mount("http://", adapter)
        self.session.mount("https://", adapter)

    def do_request(self, *args, **kwargs):
        """You may want to override this method to provide custom behavior."""
        self.external_api_proxy(self._do_request, *args, **kwargs)

    @staticmethod
    def external_api_proxy(func, *args, **kwargs):
        """Handle external service API errors.
        You may want to override this method to provide custom behavior.
        """
        error = None
        response = {"OK": True}
        try:
            func_response = func(*args, **kwargs)
            response["code"] = func_response.status_code
            response["data"] = func_response.json()
            if response["code"] >= 300:
                error = "nook"

        except json.JSONDecodeError as err:
            error = "cant_parse_json_error"
            response["data"] = func_response.text.strip()

        except (
            ConnectionError,
            requests.exceptions.RequestException,
            requests.exceptions.Timeout,
        ) as err:
            error = "connection_error"
            response["data"] = str(err)

        if error:
            response["OK"] = False
            response["error"] = error
        return response

    def _do_request(
        self,
        method: str,
        url: str,
        timeout: int = 10,
        headers: dict = None,
        payload: dict = None,
        basic: bool = None,
    ):
        options = {"url": url, "timeout": timeout}
        headers and options.update({"headers": headers})

        if payload:
            if method == "GET":
                options["params"] = payload
            elif method in ["POST", "PUT", "PATCH"] and payload:
                options["json"] = payload
        if basic:
            options["auth"] = basic
        response = getattr(self.session, method.lower())(**options)
        return response

    def get(self, url, **kwargs):
        return self.do_request(method="GET", url=url, **kwargs)

    def post(self, url, **kwargs):
        return self.do_request(method="POST", url=url, **kwargs)

    def put(self, url, **kwargs):
        return self.do_request(method="PUT", url=url, **kwargs)

    def patch(self, url, **kwargs):
        return self.do_request(method="PATCH", url=url, **kwargs)

    def delete(self, url, **kwargs):
        return self.do_request(method="DELETE", url=url, **kwargs)



from doit.action import CmdAction
from doit.tools import Interactive

dcc_params = [
    {"name": "compose_cmd", "short": "compose_cmd", "long": "compose_cmd", "type": str, "default": "docker-compose"},
    {"name": "dj_service", "short": "dj_service", "long": "dj_service", "type": str, "default": "django"},
]


def task_up():
    """ Start local docker containers. """

    def dcc(compose_cmd, dj_service):
        return f"{compose_cmd} up"

    return {"actions": [Interactive(dcc)], "verbosity": 2, "params": dcc_params, "pos_arg": "pos"}


def task_init():
    """ Initialize docker & django project. """

    return {
        "actions": [
            Interactive("cookiecutter git@bitbucket.org:appchance/pychance-cookiecutter.git"),
            CmdAction(
                (
                    "cd $(ls -td -- */ | head -n 1) && "
                    "ln -s docker/local/compose.yml ./docker-compose.yml &&"
                    "cp -r .envs/.local.example .envs/.local"
                )
            ),
            CmdAction(
                "cd $(ls -td -- */ | head -n 1) && dodo build && dodo upd && dodo manage makemigrations && dodo manage migrate && dodo logs"
            ),
        ],
        "verbosity": 2,
    }


def task_build():
    """ Build local docker images. """

    def dcc(compose_cmd, dj_service):
        return f"{compose_cmd} build"

    return {"actions": [Interactive(dcc)], "verbosity": 2, "params": dcc_params, "pos_arg": "pos"}


def task_upd():
    """ Start local docker containers in detatched mode. """

    def dcc(compose_cmd, dj_service):
        return f"{compose_cmd} up -d"

    return {"actions": [Interactive(dcc)], "verbosity": 2, "params": dcc_params, "pos_arg": "pos"}


def task_logs():
    """ Follow docker logs. """

    def dcc(compose_cmd, dj_service):
        return f"{compose_cmd} logs -f"

    return {"actions": [Interactive(dcc)], "verbosity": 2, "params": dcc_params, "pos_arg": "pos"}


def task_stop():
    """ Stop docker containers. """

    def dcc(compose_cmd, dj_service):
        return f"{compose_cmd} stop"

    return {"actions": [Interactive(dcc)], "verbosity": 2, "params": dcc_params, "pos_arg": "pos"}


def task_down():
    """ Send docker-compose down command. """

    def dcc(compose_cmd, dj_service):
        return f"{compose_cmd} down"

    return {"actions": [Interactive(dcc)], "verbosity": 2, "params": dcc_params, "pos_arg": "pos"}


def task_attach():
    """ Attach to django service. """

    def dcc(compose_cmd, dj_service):
        return f"{compose_cmd} attach {dj_service}"

    return {"actions": [Interactive(dcc)], "verbosity": 2, "params": dcc_params, "pos_arg": "pos"}


def task_pip():
    """ Run pip command. """

    def dcc(pos, compose_cmd, dj_service):
        arguments_space = " ".join(pos)
        return f"{compose_cmd} exec {dj_service} pip {pos}"

    return {"actions": [Interactive(dcc)], "verbosity": 2, "params": dcc_params, "pos_arg": "pos"}


def task_manage():
    """ Run django manage.py command with additional arguments. """

    def dcc(pos, compose_cmd, dj_service):
        return f"{compose_cmd} run --rm {dj_service} python manage.py {' '.join(pos)}"

    return {"actions": [Interactive(dcc)], "verbosity": 2, "params": dcc_params, "pos_arg": "pos"}


def task_test():
    """ Run pytest. """

    def dcc(pos, compose_cmd, dj_service):
        return f"{compose_cmd} exec {dj_service} pytest {' '.join(pos)}"

    return {"actions": [Interactive(dcc)], "verbosity": 2, "params": dcc_params, "pos_arg": "pos"}


def task_format():
    """ Format code. """

    def dcc(pos, compose_cmd, dj_service):
        return [
            f'{compose_cmd} run --rm {dj_service}   /bin/bash -c  "isort . && black -l 120 ."',
        ]

    return {"actions": [Interactive(dcc)], "verbosity": 2, "params": dcc_params, "pos_arg": "pos"}


def task_shell():
    """ Open django shell. """

    def dcc(pos, compose_cmd, dj_service):
        return f"{compose_cmd} run --rm  {dj_service} python manage.py shell_plus"

    return {"actions": [Interactive(dcc)], "verbosity": 2, "params": dcc_params, "pos_arg": "pos"}


def task_migrations():
    """ Django make & run migrations. """

    def dcc(compose_cmd, dj_service):
        return [
            f"{compose_cmd} run --rm  {dj_service} python manage.py makemigrations",
            f"{compose_cmd} run --rm  {dj_service} python manage.py migrate",
        ]

    return {"actions": [Interactive(dcc)], "verbosity": 2, "params": dcc_params}

def task_security():
    """ Run security checks. """

    def dcc_safety(compose_cmd, dj_service):
        return f"{compose_cmd} exec {dj_service} safety check"

    def dcc_bandit(compose_cmd, dj_service):
        return f"{compose_cmd} exec {dj_service} bandit --recursive ."

    def dcc_dodgy(compose_cmd, dj_service):
        return f"{compose_cmd} exec {dj_service} dodgy"


    return {"actions": [Interactive(dcc_safety), Interactive(dcc_bandit), Interactive(dcc_dodgy)], "verbosity": 2, "params": dcc_params}


import swapper
from django.db import models
from django.db.models import Prefetch


class BaseSectionQuerySet(models.QuerySet):
    def prefetch_content(self):
        SectionContent = swapper.load_model("pychance_sections", "SectionContent")

        return self.prefetch_related(
            Prefetch("content_list", queryset=SectionContent.objects.order_by("order").select_related("content"))
        )



import swapper
from rest_framework import viewsets
from rest_framework.mixins import ListModelMixin
from rest_framework.permissions import AllowAny

from appchance.sections.serializers import SectionSerializer

Section = swapper.load_model("pychance_sections", "Section")


class SectionViewSet(viewsets.GenericViewSet, ListModelMixin):
    permission_classes = (AllowAny,)
    queryset = Section.objects.all()
    serializer_class = SectionSerializer



import swapper
from django.contrib import admin

Section = swapper.load_model("pychance_sections", "Section")
SectionContent = swapper.load_model("pychance_sections", "SectionContent")
Content = swapper.load_model("pychance_sections", "Content")


class ContentListInline(admin.StackedInline):
    model = SectionContent
    extra = 1


@admin.register(Content)
class SectionContentAdmin(admin.ModelAdmin):
    inlines = (ContentListInline,)


@admin.register(Section)
class SectionAdmin(admin.ModelAdmin):
    inlines = (ContentListInline,)



# Generated by Django 3.0.10 on 2020-10-09 07:43
import django.db.models.deletion
import swapper
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        swapper.dependency("pychance_sections", "Section"),
        swapper.dependency("pychance_sections", "Content"),
        swapper.dependency("pychance_sections", "SectionContent"),
    ]

    operations = [
        migrations.CreateModel(
            name="Content",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("name", models.CharField(blank=True, max_length=255, verbose_name="name")),
                ("redirect_url", models.URLField(blank=True, null=True, verbose_name="redirect url")),
                ("content_url", models.URLField(blank=True, null=True, verbose_name="content url")),
            ],
            options={
                "swappable": swapper.swappable_setting("pychance_sections", "Content"),
            },
        ),
        migrations.CreateModel(
            name="Section",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("name", models.CharField(blank=True, max_length=255)),
                (
                    "location",
                    models.PositiveSmallIntegerField(
                        blank=True, choices=[(1, "top")], null=True, verbose_name="location"
                    ),
                ),
                (
                    "type",
                    models.PositiveSmallIntegerField(
                        blank=True, choices=[(1, "banner"), (2, "slider")], null=True, verbose_name="type"
                    ),
                ),
            ],
            options={
                "swappable": swapper.swappable_setting("pychance_sections", "Section"),
            },
        ),
        migrations.CreateModel(
            name="SectionContent",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("order", models.PositiveSmallIntegerField(default=0, verbose_name="order")),
                (
                    "content",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="content_list",
                        to=swapper.get_model_name("pychance_sections", "Content"),
                    ),
                ),
                (
                    "section",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="content_list",
                        to=swapper.get_model_name("pychance_sections", "Section"),
                    ),
                ),
            ],
            options={
                "swappable": swapper.swappable_setting("pychance_sections", "SectionContent"),
            },
        ),
    ]



from urllib.parse import quote

import requests
from django.conf import settings

from appchance.client.api import HttpClient


class ClientGraphQL(HttpClient):
    """GraphQL Client. Using GET for queries, POST for mutations"""

    def __init__(self, *args, **kwargs):
        super().__init__(self, *args, **kwargs)

    @staticmethod
    def _prepare_query(root, params, extra, mutation=False):
        _str = f"{root}"
        if params:
            _str += f"({params})"
        if extra:
            _str += extra
        _str = "{" + _str + "}"
        if mutation:
            _str = "mutation" + _str
        if settings.DEBUG:
            print(_str)
        return _str

    def query(self, root, params, extra):
        kwargs = {"basic": self.basic} if self.basic else {}  # pylint: disable=no-member
        return self.do_request(
            "GET",
            self.endpoint,  # pylint: disable=no-member
            payload={"query": self._prepare_query(root, params, extra)},
            **kwargs,
        )

    def mutation(self, root, params, extra):
        kwargs = {"basic": self.basic} if self.basic else {}  # pylint: disable=no-member
        return self.do_request(
            "POST",
            self.endpoint,  # pylint: disable=no-member
            payload={"query": self._prepare_query(root, params, extra, mutation=True)},
            **kwargs,
        )



import os
import sys

import setuptools


setuptools.setup(
    name="appchance",
    version="0.3.2.0",
    author="Appchance Backend Spellbook",
    author_email="backend@appchance.com",
    description="Appchance's spellbook for wizards and ninjas. Useful in dungeons.",
    long_description=open("README.md", "r").read(),
    long_description_content_type="text/markdown",
    url="https://bitbucket.org/appchance/pychance/",
    packages=setuptools.find_packages(),
    install_requires=["doit", "django", "cookiecutter", "ipython", "swapper"],
    scripts=["bin/dodo"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Framework :: Django :: 3.0",
        "License :: OSI Approved :: Apache Software License",
        "Development Status :: 3 - Alpha",
        "Operating System :: POSIX :: Linux",
    ],
    python_requires=">=3.7",
)



from paramiko import Transport
from paramiko.sftp_client import SFTPClient


class ClientSFTP:
    """SFTP Client"""

    def __init__(self, host, port, user, password):
        transport = Transport((host, port))
        transport.connect(None, user, password)
        self.client = SFTPClient.from_transport(transport)

    def get(self, remotepath, localpath, *args, **kwargs):
        self.client.get(remotepath, localpath, *args, **kwargs)
        self.client.close()

    def put(self, localpath, remotepath, *args, **kwargs):
        self.client.put(localpath, remotepath, *args, **kwargs)
        self.client.close()

    def get_latest(self, remotepath, localpath, *args, **kwargs):
        self.client.chdir(remotepath)
        ll = self.client.listdir_attr()
        ll.sort(key=lambda fileattr: fileattr.st_mtime)
        latest_file = ll[-1]
        self.get(latest_file.filename, localpath, *args, **kwargs)



# Generated by Django 3.1.2 on 2020-10-15 15:10

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="CustomContent",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
            ],
        ),
    ]



import swapper
from rest_framework import serializers

Section = swapper.load_model("pychance_sections", "Section")
Content = swapper.load_model("pychance_sections", "Content")
SectionContent = swapper.load_model("pychance_sections", "SectionContent")


class ContentSerializer(serializers.ModelSerializer):
    class Meta:
        model = Content
        fields = "__all__"


class SectionContentSerializer(serializers.ModelSerializer):
    content = ContentSerializer()

    class Meta:
        model = SectionContent
        fields = "__all__"


class SectionSerializer(serializers.ModelSerializer):
    content_list = SectionContentSerializer(many=True)

    class Meta:
        model = Section
        fields = "__all__"
